# cmd.sh: wrap bash functions into a command
# Copyright (C) 2012 Juan Manuel Borges Caño
# Version 0.4.4

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# The variables to fill
#cmd="[@]pkg[@]"
#cmd_name="[@]pkgname[@]"
#cmd_description="[@]pkgdescription[@]"
#cmd_version="[@]pkgversion[@]"
#cmd_author="[@]pkgauthor[@]"
#cmd_bugreport="[@]pkgbugreport[@]"
#cmd_homepage="[@]pkghomepage[@]"
#cmd_usage="[@]pkgusage[@]"
#cmd_example="[@]pkgexample[@]"
#cmd_options="[@]pkgoptions[@]"
#cmd_extrahelp="[@]pkgextrahelp[@]"
#cmd_extranotes="[@]pkgextranotes[@]"
#cmd_main="[@]pkgoptionsmain[@]"

# The usual variables
#cmd_datadir="[@]pkgdatadir[@]"

# The variables cleaned
[[ -z "$cmd" ]] && cmd="@cmd@"
[[ -z "$cmd_name" ]] && cmd_name="@name@"
[[ -z "$cmd_description" ]] && cmd_description="@description@"
[[ -z "$cmd_version" ]] && cmd_version="@version@"
[[ -z "$cmd_author" ]] && cmd_author="@author@"
[[ ! "${cmd_bugreport+_}" ]] && cmd_bugreport="@bugreport@"
[[ ! "${cmd_homepage+_}" ]] && cmd_homepage="@homepage@"
[[ -z "$cmd_usage" ]] && cmd_usage="@usage@"
[[ -z "$cmd_example" ]] && cmd_example="@example@"
[[ -z "$cmd_options" ]] && cmd_options=()
[[ ! "${cmd_extrahelp+_}" ]] && cmd_extrahelp="@extrahelp?@"
[[ ! "${cmd_extranotes+_}" ]] && cmd_extranotes="@extranotes?@"
[[ -z "$cmd_init" ]] && cmd_init=":"
[[ -z "$cmd_main" ]] && cmd_main=":"

# The info helper
function cmd_info
{
	printf "%s\n"  "Try \`$cmd --help' for more information."
}

# The warning helper
function cmd_warning
{
	printf "%s\n"  "$cmd: $@" >&2
}

# The error helper
function cmd_error
{
	cmd_warning "$@"; cmd_info >&2; exit 1
}

# The options arrays
cmd_options+=("/h/help/show a help message/cmd_help/" "/v/version/show the program version/cmd_version/")
cmd_shorts=()
cmd_longs=()
cmd_msgs=()
cmd_funcs=()
cmd_args=()
for ((i = 0; i < "${#cmd_options[@]}"; i++))
do
	IFS="${cmd_options[$i]:0:1}" read cmd_shorts[$i] cmd_longs[$i] cmd_msgs[$i] cmd_funcs[$i] cmd_args[$i] < <(printf "%s\n"  "${cmd_options[$i]:1}")
done

# The --help option
function cmd_help
{
	printf "%s\n"  "Usage: $cmd_usage"
	printf "%s\n"  "$cmd_description"
	printf "%s\n"  "Example: $cmd_example"
	printf "%s\n"  ""
	printf "%s\n"  "Mandatory arguments for long options are mandatory for short options too."
	for ((i = 0; i < "${#cmd_options[@]}"; i++))
	do
		if [[ "${cmd_shorts[$i]}" =~ ":" || "${cmd_longs[$i]}" =~ ":" ]]
		then
			printf "%-48s%s\n"  "  -${cmd_shorts[$i]//:/}, --${cmd_longs[$i]//:/}=${cmd_args[$i]}" "${cmd_msgs[$i]}"
		else
			printf "%-48s%s\n"  "  -${cmd_shorts[$i]//:/}, --${cmd_longs[$i]//:/}" "${cmd_msgs[$i]}"
		fi
	done
	[[ -n "$cmd_extrahelp" ]] && printf "%s\n"  "$cmd_extrahelp"
	printf "%s\n"  ""
	[[ -n "$cmd_bugreport" ]] && printf "%s\n"  "Report bugs to <$cmd_bugreport>."
	[[ -n "$cmd_homepage" ]] && printf "%s\n"  "$cmd ($cmd_name) home page <$cmd_homepage>."
	[[ -n "$cmd_extranotes" ]] && printf "%s\n"  "$cmd_extranotes"
	exit 0
}

# The --version option
function cmd_version
{
	printf "%s\n"  "$cmd_version"
	exit 0
}


# ... and cmd, the wrap itself
"$cmd_init" "$@"

# This handles options
cmd_s="${cmd_shorts[@]}"
cmd_l="${cmd_longs[@]}"
cmd_temp="$(getopt -o "${cmd_s// /}" -l "${cmd_l// /,}" -n "$cmd" -- "$@")"
eval set -- "$cmd_temp"
while [[ -n "$1" ]]
do
	for ((i = 0; i < "${#cmd_options[@]}"; i++))
	do
		if [[ "$1" = "-${cmd_shorts[$i]//:/}" || "$1" = "--${cmd_longs[$i]//:/}" ]]
		then
			shift
			if [[ "${cmd_shorts[$i]}" =~ ":" ]]
			then
				"${cmd_funcs[$i]}" "$1"
				shift 
			else
				"${cmd_funcs[$i]}"
			fi
			continue 2
		fi
	done
	if [[ "$1" = "--" ]]
	then
		shift
		break
	else
		cmd_error "argument error"
	fi
done

# This handles the main program
"$cmd_main" "$@"
